@@ -1,6 +1,6 @@
 package Gearman::Taskset;
 use version ();
-$Gearman::Taskset::VERSION = version->declare("2.004.003");
+$Gearman::Taskset::VERSION = version->declare("2.004.004");
 
 use strict;
 use warnings;
@@ -100,6 +100,7 @@ sub DESTROY {
             $self->{default_sock});
     }
 
+    keys %{ $self->{loaned_sock} };
     while (my ($hp, $sock) = each %{ $self->{loaned_sock} }) {
         $self->client->_sock_cache($hp, $sock);
     }
@@ -171,7 +172,7 @@ sub cancel {
         $self->{default_sock} = undef;
     }
 
-    while (my ($hp, $sock) = each %{ $self->{loaned_sock} }) {
+    foreach my $sock (values %{ $self->{loaned_sock} }) {
         $sock->close;
     }
 
@@ -373,13 +374,16 @@ sub _ip_port {
 
     # look for a hostport in loaned_sock
     my $hostport;
-    while (my ($hp, $s) = each %{ $self->{loaned_sock} }) {
+    my @k = keys %{ $self->{loaned_sock} };
+    while (!$hostport && (my $hp = shift @k)) {
+        my $s = $self->{loaned_sock}->{$hp};
         $s || next;
         if ($sock == $s) {
             $hostport = $hp;
-            last;
+
+            # last;
         }
-    } ## end while (my ($hp, $s) = each...)
+    } ## end while (!$hostport && (my ...))
 
     # hopefully it solves client->get_status mismatch
     $hostport && return $hostport;
@@ -410,8 +414,7 @@ sub _fail_jshandle {
 
     my $task = shift @{$task_list};
     (Scalar::Util::blessed($task) && $task->isa("Gearman::Task"))
-        || Carp::croak
-        "task_list is empty on $type for handle $shandle\n";
+        || Carp::croak "task_list is empty on $type for handle $shandle\n";
 
     $task->fail($msg || "jshandle fail");
 
@@ -480,10 +483,8 @@ sub process_packet {
 
             my $task_list = $self->{waiting}{$shandle};
             my $task      = $task_list->[0];
-            $assert{task}->(
-                $task,
-                "task_list is empty on work_data for handle $shandle"
-            );
+            $assert{task}->($task,
+                "task_list is empty on work_data for handle $shandle");
 
             $task->data(\$blob);
 
@@ -498,8 +499,7 @@ sub process_packet {
             my $task_list = $self->{waiting}{$shandle};
             my $task      = $task_list->[0];
             $assert{task}->(
-                $task,
-                "task_list is empty on work_warning for handle $shandle"
+                $task, "task_list is empty on work_warning for handle $shandle"
             );
 
             $task->warning(\$blob);
@@ -538,8 +538,7 @@ sub process_packet {
             my ($shandle, $nu, $de) = split(/\0/, $blob);
             my $task_list = $self->{waiting}{$shandle};
             ref($task_list) eq "ARRAY" && scalar(@{$task_list})
-                or Carp::croak
-                "Got work_status for unknown handle: $shandle";
+                or Carp::croak "Got work_status for unknown handle: $shandle";
 
             # FIXME: the server is (probably) sending a work_status packet for each
             # interested client, even if the clients are the same, so probably need
